'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const teletype_client_1 = require("@atom/teletype-client");
const PortalBinding_1 = require("./PortalBinding");
const fetch = require('node-fetch');
const constants = require('./constants');
const globalAny = global;
const wrtc = require('wrtc');
globalAny.window = {};
globalAny.window = global;
globalAny.window.fetch = fetch;
globalAny.RTCPeerConnection = wrtc.RTCPeerConnection;
// this method is called when the extension is activated
// the extension is activated the very first time the command is executed
function activate(context) {
    console.log('Great, your extension "vscode-teletype" is now active!');
    let disposable = vscode.commands.registerCommand('extension.join-portal', () => __awaiter(this, void 0, void 0, function* () {
        let portalIdInput = yield getPortalID();
        if (!portalIdInput) {
            vscode.window.showInformationMessage("No Portal ID has been entered. Please try again");
        }
        else {
            vscode.window.showInformationMessage('Trying to Join Portal with ID' + ' ' + portalIdInput + ' ');
            yield joinPortal(portalIdInput);
        }
    }));
    context.subscriptions.push(disposable);
}
exports.activate = activate;
function getPortalID() {
    return __awaiter(this, void 0, void 0, function* () {
        let portalID = yield vscode.window.showInputBox({ prompt: 'Enter ID of the Portal you wish to join' });
        return portalID;
    });
}
function joinPortal(portalId) {
    return __awaiter(this, void 0, void 0, function* () {
        let textEditor = vscode.window.activeTextEditor;
        let client, portal_binding;
        if (constants.AUTH_TOKEN !== '') {
            try {
                client = new teletype_client_1.TeletypeClient({
                    pusherKey: constants.PUSHER_KEY,
                    pusherOptions: {
                        cluster: constants.PUSHER_CLUSTER
                    },
                    baseURL: constants.API_URL_BASE,
                });
                yield client.initialize();
                yield client.signIn(constants.AUTH_TOKEN);
            }
            catch (e) {
                console.log("Exception Error Message " + e);
            }
            portal_binding = new PortalBinding_1.default({ client: client, portalId: portalId, editor: textEditor });
            yield portal_binding.initialize();
        }
        else {
            vscode.window.showErrorMessage("GitHub Auth Token. Please provide it in the constants.ts file");
        }
    });
}
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map